@@ -25,14 +25,22 @@ import mockintosh
 from mockintosh.constants import PROGRAM, SUPPORTED_ENGINES, PYBARS, JINJA, SPECIAL_CONTEXT
 from mockintosh.exceptions import UnsupportedTemplateEngine
 from mockintosh.templating import TemplateRenderer
-from mockintosh.params import PathParam, HeaderParam, QueryStringParam
+from mockintosh.params import PathParam, HeaderParam, QueryStringParam, BodyParam
 from mockintosh.methods import _safe_path_split, _detect_engine
 
+from mockintosh.hbs.methods import Random as hbs_Random
+from mockintosh.j2.methods import Random as j2_Random
+
 OPTIONS = 'options'
 ORIGIN = 'Origin'
 AC_REQUEST_HEADERS = 'Access-Control-Request-Headers'
 NON_PREFLIGHT_METHODS = ("GET", "HEAD", "POST", "DELETE", "PATCH", "PUT")
 
+hbs_random = hbs_Random()
+j2_random = j2_Random()
+
+counters = {}
+
 
 class GenericHandler(tornado.web.RequestHandler):
 
@@ -44,6 +52,7 @@ class GenericHandler(tornado.web.RequestHandler):
         self.definition_engine = definition_engine
         self.interceptors = interceptors
         self.is_options = False
+        self.custom_dataset = {}
 
         self.special_request = self.build_special_request()
         self.default_context = {
@@ -58,19 +67,22 @@ class GenericHandler(tornado.web.RequestHandler):
             self.dynamic_unimplemented_method_guard()
 
         try:
-            _id, response, params, context = self.match_alternative()
+            _id, response, params, context, dataset = self.match_alternative()
         except TypeError:
             return
         self.custom_endpoint_id = _id
         self.custom_response = response
         self.custom_params = params
         self.initial_context = context
+        self.custom_dataset = dataset
 
         self.populate_context(*args)
         self.determine_status_code()
         self.determine_headers()
         self.log_request()
+
         self.rendered_body = self.render_template()
+
         if self.rendered_body is None:
             return
         self.special_response = self.build_special_response()
@@ -101,15 +113,27 @@ class GenericHandler(tornado.web.RequestHandler):
 
     def populate_context(self, *args):
         self.custom_context = {}
+        for key, value in self.custom_dataset.items():
+            self.custom_context[key] = value
         if args:
             if len(args) >= len(self.initial_context):
                 for i, key in enumerate(self.initial_context):
                     self.custom_context[key] = args[i]
             else:
-                HTTPError(400)
+                HTTPError(404)
         self.custom_context.update(self.default_context)
         self.analyze_headers()
         self.analyze_query_string()
+        self.analyze_body_text()
+        self.analyze_counters()
+
+    def populate_counters(self, context):
+        if context is None:
+            return
+
+        if SPECIAL_CONTEXT in context and 'counters' in context[SPECIAL_CONTEXT]:
+            for key, value in context[SPECIAL_CONTEXT]['counters'].items():
+                counters[key] = value
 
     def dynamic_unimplemented_method_guard(self):
         if self.request.method.lower() not in self.methods:
@@ -128,6 +152,8 @@ class GenericHandler(tornado.web.RequestHandler):
                 context[key] = self.request.headers.get(param.key.title())
             if isinstance(param, QueryStringParam):
                 context[key] = self.get_query_argument(param.key)
+            if isinstance(param, BodyParam):
+                context[key] = self.decoder(self.request.body)
         return context
 
     def render_template(self):
@@ -153,13 +179,16 @@ class GenericHandler(tornado.web.RequestHandler):
                     logging.debug('Template file is binary. Templating disabled.')
 
         compiled = None
+        context = None
         if is_binary or not self.custom_response.get('useTemplating', True):
             compiled = source_text
         else:
             if template_engine == PYBARS:
-                from mockintosh.hbs.methods import uuid, fake, random_integer
+                from mockintosh.hbs.methods import fake, counter, json_path
+                self.custom_context['random'] = hbs_random
             elif template_engine == JINJA:
-                from mockintosh.j2.methods import uuid, fake, random_integer
+                from mockintosh.j2.methods import fake, counter, json_path
+                self.custom_context['random'] = j2_random
             else:
                 raise UnsupportedTemplateEngine(template_engine, SUPPORTED_ENGINES)
 
@@ -168,13 +197,15 @@ class GenericHandler(tornado.web.RequestHandler):
                 source_text,
                 inject_objects=self.custom_context,
                 inject_methods=[
-                    uuid,
                     fake,
-                    random_integer
+                    counter,
+                    json_path
                 ],
-                add_params_callback=self.add_params
+                add_params_callback=self.add_params,
+                fill_undefineds=True
             )
-            compiled, _ = renderer.render()
+            compiled, context = renderer.render()
+            self.populate_counters(context)
 
         if not is_binary:
             logging.debug('Render output: %s' % compiled)
@@ -246,14 +277,25 @@ class GenericHandler(tornado.web.RequestHandler):
         status_code = None
         if 'status' in self.custom_response:
             if isinstance(self.custom_response['status'], str):
+                if self.definition_engine == PYBARS:
+                    from mockintosh.hbs.methods import counter
+                elif self.definition_engine == JINJA:
+                    from mockintosh.j2.methods import counter
+                else:
+                    raise UnsupportedTemplateEngine(self.definition_engine, SUPPORTED_ENGINES)
+
                 renderer = TemplateRenderer(
                     self.definition_engine,
                     self.custom_response['status'],
                     inject_objects=self.custom_context,
-                    inject_methods=[],
-                    add_params_callback=self.add_params
+                    inject_methods=[
+                        counter
+                    ],
+                    add_params_callback=self.add_params,
+                    fill_undefineds=True
                 )
-                compiled, _ = renderer.render()
+                compiled, context = renderer.render()
+                self.populate_counters(context)
                 status_code = int(compiled)
             else:
                 status_code = self.custom_response['status']
@@ -268,7 +310,7 @@ class GenericHandler(tornado.web.RequestHandler):
         for header_key, header in self.initial_context[SPECIAL_CONTEXT]['headers'].items():
             if header_key.title() in self.request.headers._dict:
                 if header['type'] == 'regex':
-                    match = re.match(header['regex'], self.request.headers.get(header_key))
+                    match = re.search(header['regex'], self.request.headers.get(header_key))
                     if match is not None:
                         for i, key in enumerate(header['args']):
                             self.custom_context[key] = match.group(i + 1)
@@ -280,11 +322,27 @@ class GenericHandler(tornado.web.RequestHandler):
         for key, value in self.initial_context[SPECIAL_CONTEXT]['queryString'].items():
             if key in self.request.query_arguments:
                 if value['type'] == 'regex':
-                    match = re.match(value['regex'], self.get_query_argument(key))
+                    match = re.search(value['regex'], self.get_query_argument(key))
                     if match is not None:
                         for i, key in enumerate(value['args']):
                             self.custom_context[key] = match.group(i + 1)
 
+    def analyze_body_text(self):
+        if SPECIAL_CONTEXT not in self.initial_context or 'bodyText' not in self.initial_context[SPECIAL_CONTEXT]:
+            return
+
+        body = self.decoder(self.request.body)
+        for key, value in self.initial_context[SPECIAL_CONTEXT]['bodyText'].items():
+            if value['type'] == 'regex':
+                match = re.search(value['regex'], body)
+                if match is not None:
+                    for i, key in enumerate(value['args']):
+                        self.custom_context[key] = match.group(i + 1)
+
+    def analyze_counters(self):
+        for key, value in counters.items():
+            self.custom_context[key] = value
+
     def determine_headers(self):
         if self.custom_endpoint_id is not None:
             self.set_header('x-%s-endpoint-id' % PROGRAM.lower(), self.custom_endpoint_id)
@@ -296,6 +354,13 @@ class GenericHandler(tornado.web.RequestHandler):
         if 'headers' not in self.custom_response:
             return
 
+        if self.definition_engine == PYBARS:
+            from mockintosh.hbs.methods import counter
+        elif self.definition_engine == JINJA:
+            from mockintosh.j2.methods import counter
+        else:
+            raise UnsupportedTemplateEngine(self.definition_engine, SUPPORTED_ENGINES)
+
         for key, value in self.custom_response['headers'].items():
             value_list = None
             if isinstance(value, list):
@@ -310,10 +375,14 @@ class GenericHandler(tornado.web.RequestHandler):
                     self.definition_engine,
                     value,
                     inject_objects=self.custom_context,
-                    inject_methods=[],
-                    add_params_callback=self.add_params
+                    inject_methods=[
+                        counter
+                    ],
+                    add_params_callback=self.add_params,
+                    fill_undefineds=True
                 )
-                new_value, _ = renderer.render()
+                new_value, context = renderer.render()
+                self.populate_counters(context)
                 new_value_list.append(new_value)
 
             for value in new_value_list:
@@ -348,7 +417,7 @@ class GenericHandler(tornado.web.RequestHandler):
                     if value == request_header_val:
                         continue
                     value = '^%s$' % value
-                    match = re.match(value, request_header_val)
+                    match = re.search(value, request_header_val)
                     if match is None:
                         fail = True
                         break
@@ -370,7 +439,7 @@ class GenericHandler(tornado.web.RequestHandler):
                     if value == request_query_val:
                         continue
                     value = '^%s$' % value
-                    match = re.match(value, request_query_val)
+                    match = re.search(value, request_query_val)
                     if match is None:
                         fail = True
                         break
@@ -379,6 +448,9 @@ class GenericHandler(tornado.web.RequestHandler):
 
             # Body
             if 'body' in alternative:
+                body = self.decoder(self.request.body)
+
+                # Schema
                 if 'schema' in alternative['body']:
                     json_schema = alternative['body']['schema']
                     if isinstance(json_schema, str) and len(json_schema) > 1 and json_schema[0] == '@':
@@ -387,7 +459,6 @@ class GenericHandler(tornado.web.RequestHandler):
                             logging.info('Reading JSON schema file from path: %s' % json_schema_path)
                             json_schema = json.load(file)
                             logging.debug('JSON schema: %s' % json_schema)
-                    body = self.request.body.decode('utf-8')
                     json_data = None
 
                     if body and json_schema:
@@ -404,10 +475,19 @@ class GenericHandler(tornado.web.RequestHandler):
                             fail = True
                             break
 
+                # Text
+                if 'text' in alternative['body']:
+                    value = alternative['body']['text']
+                    if not body == value:
+                        match = re.search(value, body)
+                        if match is None:
+                            fail = True
+                            break
+
             if self.should_cors():
                 self.respond_cors()
 
-            _id = alternative['id']
+            # Multiple responses
             if 'response' in alternative:
                 response = alternative['response']
                 if isinstance(alternative['response'], list):
@@ -416,19 +496,7 @@ class GenericHandler(tornado.web.RequestHandler):
                             'body': None
                         }
                     else:
-                        if 'multiResponsesIndex' not in alternative:
-                            alternative['multiResponsesIndex'] = 0
-                        else:
-                            alternative['multiResponsesIndex'] += 1
-
-                        if alternative['multiResponsesIndex'] > len(alternative['response']) - 1:
-                            if alternative.get('multiResponsesLooped', True):
-                                alternative['multiResponsesIndex'] = 0
-                            else:
-                                self.set_status(410)
-                                self.finish()
-                                return
-                        response = alternative['response'][alternative['multiResponsesIndex']]
+                        response = self.loop_alternative(alternative, 'response', 'multiResponses')
 
                 response = response if isinstance(response, dict) else {
                     'body': response
@@ -437,11 +505,21 @@ class GenericHandler(tornado.web.RequestHandler):
                 response = {
                     'body': None
                 }
+
+            # Dataset
+            dataset = {}
+            if 'dataset' in alternative:
+                alternative['dataset'] = self.load_dataset(alternative['dataset'])
+                if alternative['dataset']:
+                    dataset = self.loop_alternative(alternative, 'dataset', 'dataset')
+
+            _id = alternative['id']
             params = alternative['params']
             context = alternative['context']
-            return _id, response, params, context
+            return _id, response, params, context, dataset
 
-        self.set_status(404)
+        if not self.__class__.__name__ == 'ErrorHandler':
+            self.set_status(400)
         self.finish()
 
     def trigger_interceptors(self):
@@ -470,14 +548,25 @@ class GenericHandler(tornado.web.RequestHandler):
         relative_path = None
         orig_relative_path = source_text[1:]
 
+        if self.definition_engine == PYBARS:
+            from mockintosh.hbs.methods import counter
+        elif self.definition_engine == JINJA:
+            from mockintosh.j2.methods import counter
+        else:
+            raise UnsupportedTemplateEngine(self.definition_engine, SUPPORTED_ENGINES)
+
         renderer = TemplateRenderer(
             self.definition_engine,
             orig_relative_path,
             inject_objects=self.custom_context,
-            inject_methods=[],
-            add_params_callback=self.add_params
+            inject_methods=[
+                counter
+            ],
+            add_params_callback=self.add_params,
+            fill_undefineds=True
         )
-        orig_relative_path, _ = renderer.render()
+        orig_relative_path, context = renderer.render()
+        self.populate_counters(context)
 
         error_msg = 'External template file \'%s\' couldn\'t be accessed or found!' % orig_relative_path
         if orig_relative_path[0] == '/':
@@ -530,6 +619,38 @@ class GenericHandler(tornado.web.RequestHandler):
             self.is_options and self.request.method.lower() not in self.methods.keys()
         )
 
+    def load_dataset(self, dataset):
+        if isinstance(dataset, list):
+            return dataset
+        else:
+            dataset_path = self.resolve_relative_path(dataset)
+            with open(dataset_path, 'r') as file:
+                logging.info('Reading dataset file from path: %s' % dataset_path)
+                data = json.load(file)
+                logging.debug('Dataset: %s' % data)
+                return data
+
+    def loop_alternative(self, alternative, key, subkey):
+        index_key = '%sIndex' % subkey
+        loop_key = '%sLooped' % subkey
+        if index_key not in alternative:
+            alternative[index_key] = 0
+        else:
+            alternative[index_key] += 1
+
+        if alternative[index_key] > len(alternative[key]) - 1:
+            if alternative.get(loop_key, True):
+                alternative[index_key] = 0
+            else:
+                self.set_status(410)
+                self.finish()
+                return
+        return alternative[key][alternative[index_key]]
+
+
+class NotParsedJSON():
+    pass
+
 
 class Request():
 
@@ -545,9 +666,20 @@ class Request():
         self.headers = {}
         self.queryString = {}
         self.body = None
+        self._json = NotParsedJSON()
         self.files = {}
         self.formData = {}
 
+    @property
+    def json(self):
+        if isinstance(self._json, NotParsedJSON):
+            try:
+                self._json = json.loads(self.body)
+            except json.JSONDecodeError:
+                logging.warning('Failed to decode request body to JSON:\n%s' % self.body)
+                self._json = None
+        return self._json
+
 
 class Response():
 
