@@ -1,5 +1,6 @@
 import logging
 import os
+import re
 import time
 import unittest
 
@@ -9,8 +10,9 @@ import mockintosh
 
 SRV1 = os.environ.get('SRV1', 'http://localhost:8001')
 SRV2 = os.environ.get('SRV2', 'http://localhost:8002')
-SRV3 = os.environ.get('SRV3', 'http://localhost:8003')
+SRV3 = os.environ.get('SRV3', 'https://localhost:8003')
 SRV4 = os.environ.get('SRV4', 'http://localhost:8004')
+SRV5 = os.environ.get('SRV5', 'https://localhost:8005')
 
 
 class IntegrationTests(unittest.TestCase):
@@ -87,15 +89,15 @@ class IntegrationTests(unittest.TestCase):
 
         path = '/qstr-matching1?param1=constantval&param2=%s&param3=prefix-%s-suffix' % (param2, param3)
         resp = requests.get(SRV1 + path)
-        self.assertEqual(404, resp.status_code)
+        self.assertEqual(400, resp.status_code)
 
         path = '/qstr-matching1?param1=constant%%20val&param22=%s&param3=prefix-%s-suffix' % (param2, param3)
         resp = requests.get(SRV1 + path)
-        self.assertEqual(404, resp.status_code)
+        self.assertEqual(400, resp.status_code)
 
         path = '/qstr-matching1?param1=constant%%20val&param2=%s&param3=prefix-%s-sufix' % (param2, param3)
         resp = requests.get(SRV1 + path)
-        self.assertEqual(404, resp.status_code)
+        self.assertEqual(400, resp.status_code)
 
     def test_headers(self):
         param2 = str(int(time.time()))
@@ -109,11 +111,11 @@ class IntegrationTests(unittest.TestCase):
 
         resp = requests.get(SRV1 + path,
                             headers={"hdr1": "constant", "hdr2": param2, "hdr3": "prefix-%s-suffix" % param3})
-        self.assertEqual(404, resp.status_code)
+        self.assertEqual(400, resp.status_code)
 
         resp = requests.get(SRV1 + path,
                             headers={"hdr1": "constant val", "hdr2": param2, "hdr3": "prefics-%s-suffix" % param3})
-        self.assertEqual(404, resp.status_code)
+        self.assertEqual(400, resp.status_code)
 
         resp = requests.get(SRV1 + path,
                             headers={"hdr4": "another header"})
@@ -127,7 +129,7 @@ class IntegrationTests(unittest.TestCase):
 
         path = '/body-jsonschema1'
         resp = requests.post(SRV1 + path, json={"somekey2": "invalid"})
-        self.assertEqual(404, resp.status_code)
+        self.assertEqual(400, resp.status_code)
 
     def test_status_templated(self):
         path = '/status-template1'
@@ -136,7 +138,7 @@ class IntegrationTests(unittest.TestCase):
 
     def test_interceptor(self):
         path = '/interceptor-modified'
-        resp = requests.get(SRV3 + path)
+        resp = requests.get(SRV3 + path, verify=False)
         self.assertEqual(202, resp.status_code)
         self.assertEqual("intercepted", resp.text)
         self.assertEqual("some-i-val", resp.headers.get("someheader"))
@@ -214,3 +216,103 @@ class IntegrationTests(unittest.TestCase):
 
         resp = requests.get(SRV1 + '/multi-response-nonlooped')
         self.assertEqual(410, resp.status_code)
+
+    def test_dataset(self):
+        resp = requests.get(SRV1 + '/dataset-inline')
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual("dset: val1", resp.text)
+
+        resp = requests.get(SRV1 + '/dataset-inline')
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual("dset: val2", resp.text)
+
+        resp = requests.get(SRV1 + '/dataset-inline')
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual("dset: val1", resp.text)
+
+    def test_dataset_fromfile(self):
+        resp = requests.get(SRV1 + '/dataset-fromfile')
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual("dset: val3", resp.text)
+
+        resp = requests.get(SRV1 + '/dataset-fromfile')
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual("dset: val4", resp.text)
+
+        resp = requests.get(SRV1 + '/dataset-fromfile')
+        self.assertEqual(410, resp.status_code)
+
+    def test_ssl(self):
+        resp = requests.get(SRV5 + '/', verify=False)
+        self.assertEqual(200, resp.status_code)
+
+    def test_templating_random(self):
+        resp = requests.get(SRV1 + '/templating-random')
+        self.assertEqual(200, resp.status_code)
+        resp = resp.text
+
+        rint, resp = resp.split("\n", 1)
+        self.assertTrue(10 <= int(rint) <= 20)
+
+        rfloat, resp = resp.split("\n", 1)
+        self.assertTrue(-0.5 <= float(rfloat) <= 20.7)
+        self.assertTrue(1 <= len(rfloat.split('.')[1]) <= 3)
+
+        alphanum, resp = resp.split("\n", 1)
+        self.assertEqual(5, len(alphanum))
+        self.assertTrue(alphanum.isalnum())
+
+        fhex, resp = resp.split("\n", 1)
+        self.assertEqual(16, len(fhex))
+        self.assertTrue(re.match("[0-9a-f]+", fhex))
+
+        fuuid, resp = resp.split("\n", 1)
+        self.assertTrue(re.match(r'[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}', fuuid))
+
+        self.assertEqual(5, len(resp))  # random ascii
+
+    def test_body_regex(self):
+        resp = requests.post(SRV1 + '/body-regex', data="somewhere 1-required-2 is present")
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual("body regex matched: 1 2", resp.text)
+
+        resp = requests.post(SRV1 + '/body-regex', data="somewhere a-required-b is not present")
+        self.assertEqual(400, resp.status_code)
+
+    def test_body_jsonpath(self):
+        resp = requests.post(SRV1 + '/body-jsonpath-tpl', json={"key": "val", "key2": 123})
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual('body jsonpath matched: val 123', resp.text)
+
+        resp = requests.post(SRV1 + '/body-jsonpath-tpl', json={"key": None})
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual('body jsonpath matched: null ', resp.text)
+
+        resp = requests.post(SRV1 + '/body-jsonpath-tpl', data="not json")
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual("body jsonpath matched: {{jsonPath request.json '$.key'}} {{jsonPath request.json '$.key2'}}",
+                         resp.text)
+
+    def test_counter(self):
+        resp = requests.get(SRV1 + '/counter1')
+        self.assertEqual("variant1: 1 1", resp.text)
+
+        resp = requests.get(SRV1 + '/counter1')
+        self.assertEqual("variant2: 2 1", resp.text)
+
+        resp = requests.get(SRV1 + '/counter1')
+        self.assertEqual("variant1: 3 2", resp.text)
+
+        resp = requests.get(SRV1 + '/counter1')
+        self.assertEqual("variant2: 4 2", resp.text)
+
+        resp = requests.get(SRV1 + '/counter2')
+        self.assertEqual("variant3: 5 3 3", resp.headers.get('X-Counter'))
+
+        resp = requests.get(SRV1 + '/counter3')
+        self.assertEqual("variant3: 5 3 3", resp.text)
+
+    def test_unprocessed_templates(self):
+        resp = requests.get(SRV1 + '/undefined-templates')
+        self.assertEqual("here goes {{unknownUndefined}} var", resp.text)
+        self.assertEqual("also {{random.intt 10 20}} can happen", resp.headers.get('X-header'))
